@@ -9,12 +9,11 @@
 import torch
 from torch.nn import Module
 
-from ..attention_registry import AttentionRegistry, Optional, Callable
-from ..causal_product import causal_dot_product 
-
-
-def elu_feature_map(x):
-    return torch.nn.functional.elu(x) + 1
+from ..attention_registry import AttentionRegistry, Optional, Callable, Int, \
+    EventDispatcherInstance
+from ..events import EventDispatcher
+from ..causal_product import causal_dot_product
+from ..feature_maps import elu_feature_map
 
 
 def causal_linear(Q, K, V):
@@ -41,17 +40,40 @@ class CausalLinearAttention(Module):
                      last dimension of a tensor (default: elu(x)+1)
         eps: float, a small number to ensure the numerical stability of the
              denominator (default: 1e-6)
+        event_dispatcher: str or EventDispatcher instance to be used by this
+                          module for dispatching events (default: the default
+                          global dispatcher)
     """
-    def __init__(self, feature_map=None, eps=1e-6):
+    def __init__(self, query_dimensions, feature_map=None, eps=1e-6,
+                 event_dispatcher=""):
         super(CausalLinearAttention, self).__init__()
-        self.feature_map = feature_map or elu_feature_map
+        self.feature_map = (
+            feature_map(query_dimensions) if feature_map else
+            elu_feature_map(query_dimensions)
+        )
         self.eps = eps
+        self.event_dispatcher = EventDispatcher.get(event_dispatcher)
+
+    def _make_sizes_compatible(self, Q, K):
+        """Either slice or pad K in case that the sizes do not match between Q
+        and K."""
+        N, L, H, E = Q.shape
+        _, S, _, _ = K.shape
+        if L == S:
+            return Q, K
+
+        if L < S:
+            return Q, K[:, :L, :, :]
+
+        if L > S:
+            return Q, torch.cat([K, K.new_zeros(N, L-S, H, E)], dim=1)
 
     def forward(self, queries, keys, values, attn_mask, query_lengths,
                 key_lengths):
         # Apply the feature map to the queries and keys
-        Q = self.feature_map(queries)
-        K = self.feature_map(keys)
+        self.feature_map.new_feature_map()
+        Q = self.feature_map.forward_queries(queries)
+        K = self.feature_map.forward_keys(keys)
 
         # Apply the key padding mask and make sure the attn_mask is a
         # lower triangular causal mask
@@ -60,6 +82,10 @@ class CausalLinearAttention(Module):
                                 "lower triangular masks"))
         K = K * key_lengths.float_matrix[:, :, None, None]
 
+        # Ensure that Q and K have compatible sizes for the following
+        # computations, namely L == S
+        Q, K = self._make_sizes_compatible(Q, K)
+
         # TODO: Shall we divide the Q and K with a relatively large number to
         #       avoid numerical instabilities in computing the denominator?
         #       We used to divide each with the max norm of all q and k but
@@ -82,5 +108,9 @@ class CausalLinearAttention(Module):
 # builders
 AttentionRegistry.register(
     "causal-linear", CausalLinearAttention,
-    [("feature_map", Optional(Callable))]
+    [
+        ("query_dimensions", Int),
+        ("feature_map", Optional(Callable)),
+        ("event_dispatcher", Optional(EventDispatcherInstance, ""))
+    ]
 )
