@@ -9,11 +9,10 @@
 import torch
 from torch.nn import Module
 
-from ..attention_registry import AttentionRegistry, Optional, Callable
-
-
-def elu_feature_map(x):
-    return torch.nn.functional.elu(x) + 1
+from ..attention_registry import AttentionRegistry, Optional, Callable, Int, \
+    EventDispatcherInstance
+from ..events import EventDispatcher
+from ..feature_maps import elu_feature_map
 
 
 class LinearAttention(Module):
@@ -39,17 +38,26 @@ class LinearAttention(Module):
                      last dimension of a tensor (default: elu(x)+1)
         eps: float, a small number to ensure the numerical stability of the
              denominator (default: 1e-6)
+        event_dispatcher: str or EventDispatcher instance to be used by this
+                          module for dispatching events (default: the default
+                          global dispatcher)
     """
-    def __init__(self, feature_map=None, eps=1e-6):
+    def __init__(self, query_dimensions, feature_map=None, eps=1e-6,
+                 event_dispatcher=""):
         super(LinearAttention, self).__init__()
-        self.feature_map = feature_map or elu_feature_map
+        self.feature_map = (
+            feature_map(query_dimensions) if feature_map else
+            elu_feature_map(query_dimensions)
+        )
         self.eps = eps
+        self.event_dispatcher = EventDispatcher.get(event_dispatcher)
 
     def forward(self, queries, keys, values, attn_mask, query_lengths,
                 key_lengths):
         # Apply the feature map to the queries and keys
-        Q = self.feature_map(queries)
-        K = self.feature_map(keys)
+        self.feature_map.new_feature_map()
+        Q = self.feature_map.forward_queries(queries)
+        K = self.feature_map.forward_keys(keys)
 
         # Apply the key padding mask and make sure that the attn_mask is
         # all_ones
@@ -76,5 +84,9 @@ class LinearAttention(Module):
 # builders
 AttentionRegistry.register(
     "linear", LinearAttention,
-    [("feature_map", Optional(Callable))]
+    [
+        ("query_dimensions", Int),
+        ("feature_map", Optional(Callable)),
+        ("event_dispatcher", Optional(EventDispatcherInstance, ""))
+    ]
 )
